# -*- coding: utf-8 -*-
# Folder Color 0.0.26
# Copyright (C) 2012-2014 Marcos Alvarez Costales https://launchpad.net/~costales
#
# Folder Color is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
# 
# Folder Color is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Folder Color; if not, see http://www.gnu.org/licenses
# for more information.

import os, urllib, gettext, locale, webbrowser, glob
from gi.repository import Gtk, Nautilus, GObject, Gio, GLib

# i18n
locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain('folder-color')
gettext.textdomain('folder-color')
_ = gettext.gettext


class ChangeColorFolder(GObject.GObject, Nautilus.MenuProvider):
    def __init__(self):
        dummy_default_i18n = (_("Black"), _("Brown"), _("Cyan"), _("Blue"), _("Green"), _("Grey"), _("Orange"), _("Pink"), _("Red"), _("Purple"), _("Yellow"), _("White")) # Need i18n, because colors are dynamic now
        if not len(self.get_colors()):
            return # No icons = Nothing to do
        
        self.DEFAULT_THEME_PATH = '/usr/share/icons/hicolor/scalable/places'
        self.SEPARATOR = "".join(["  ", u'\u2015'])
        self.GRADIENT_MIDDLE_RANGE = 2500
        self.GRADIENT_DARK_RANGE = 5000
        self.HIDE_DONATION = os.path.join(os.getenv("HOME"), '.config', 'folder-color', 'hide_donation')
        self.FOLDER_ICONS_CUSTOM = os.path.join(os.getenv("HOME"), '.config', 'folder-color', 'custom_icons/')
        self.DEFAULT_FOLDERS = self.get_default_folders()
        
        if not os.path.exists(self.FOLDER_ICONS_CUSTOM):
            try:
                os.makedirs(self.FOLDER_ICONS_CUSTOM)
            except OSError as exception:
                pass
            except:
                pass
        pass
    
    def get_theme_path(self):
        icon_theme = Gtk.IconTheme.get_default()
        icon_info = icon_theme.lookup_icon("folder_color_custom", 0, 0)
        try:
            icon_path = icon_info.get_filename()
        except:
            icon_path = self.DEFAULT_THEME_PATH
        return icon_path.replace('/folder_color_custom.svg', '')
    
    def get_colors(self):
        available_colors = {}
        available_icons = glob.glob(os.path.join(self.get_theme_path(), 'folder_color_*.svg'))
        for each_icon in available_icons:
            each_icon = os.path.basename(each_icon)
            if not any(not_count in each_icon for not_count in ('picker', 'custom', 'desktop', 'documents', 'downloads', 'music', 'pictures', 'public', 'templates', 'videos')):
                color = each_icon.replace('folder_color_', '').replace('.svg', '').replace('_', ' ')
                available_colors[color] = _(color.title())
        return available_colors
    
    def get_default_folders(self):
        folders = {}
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_DESKTOP)]      = 'desktop'
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_DOCUMENTS)]    = 'documents'
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_DOWNLOAD)]     = 'downloads'
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_MUSIC)]        = 'music'
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_PICTURES)]     = 'pictures'
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_PUBLIC_SHARE)] = 'public'
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_TEMPLATES)]    = 'templates'
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_VIDEOS)]       = 'videos'
        return folders
    
    def get_icon_name(self, folder, color):
        color = color.replace(' ', '_') # Come from the file name
        
        # Available preconfigured color?
        try:
            if os.path.isfile(os.path.join(self.get_theme_path(), "folder_color_%s_%s.svg" % (color, self.DEFAULT_FOLDERS[folder]))):
                return "_".join(['folder', 'color', color, self.DEFAULT_FOLDERS[folder]])
        except:
            pass
        
        # Default color
        return "_".join(['folder', 'color', color])
    
    def choose_custom_color(self):
        colorselectiondialog = Gtk.ColorSelectionDialog(_("Choose a Folder Color"))
        selector = colorselectiondialog.get_color_selection()
        response = colorselectiondialog.run()
        colorselectiondialog.destroy()
        
        if response == Gtk.ResponseType.OK:
            color = selector.get_current_color()
            # Control max color range
            if color.red <= self.GRADIENT_DARK_RANGE:
                color.red = self.GRADIENT_DARK_RANGE
            if color.green <= self.GRADIENT_DARK_RANGE:
                color.green = self.GRADIENT_DARK_RANGE
            if color.blue <= self.GRADIENT_DARK_RANGE:
                color.blue = self.GRADIENT_DARK_RANGE
            # Convert to hex
            rgb_ligth = (color.red, color.green, color.blue)
            rgb_middle = (color.red-self.GRADIENT_MIDDLE_RANGE, color.green-self.GRADIENT_MIDDLE_RANGE, color.blue-self.GRADIENT_MIDDLE_RANGE)
            rgb_dark = (color.red-self.GRADIENT_DARK_RANGE, color.green-self.GRADIENT_DARK_RANGE, color.blue-self.GRADIENT_DARK_RANGE)
            htmlcolor_light =  ''.join((str(hex(i/257))[2:].rjust(2, '0') for i in rgb_ligth)).upper()
            htmlcolor_middle = ''.join((str(hex(i/257))[2:].rjust(2, '0') for i in rgb_middle)).upper()
            htmlcolor_dark =   ''.join((str(hex(i/257))[2:].rjust(2, '0') for i in rgb_dark)).upper()
            return (htmlcolor_light, htmlcolor_middle, htmlcolor_dark)
        
        return None
    
    def generate_icon_from_template(self, colors):
        # Parsing template to a custom color
        file_custom = self.FOLDER_ICONS_CUSTOM + colors[0] + '.svg'
        f_input  = open(os.path.join(self.get_theme_path(), 'folder_color_custom.svg'), 'r') # This has to exists always
        f_output = open(file_custom, 'w')
        
        f_generate  = f_input.read().replace("value_light", colors[0]).replace("value_middle", colors[1]).replace("value_dark", colors[2])
        f_output.write(f_generate)
        
        return file_custom
    
    def reload_icon(self, folder):
        # Reload the current folder
        os.system("touch \"%s\"" % folder)
    
    def menu_activate_cb(self, menu, color, folders):
        # Donation
        if color == 'donate':
            webbrowser.open_new("http://gufw.org/donate_foldercolor")
            return
        # Hide donation
        if color == 'hideDonate':
            try:
                os.makedirs(self.HIDE_DONATION)
            except OSError as exception:
                pass
            except:
                pass
            return
        
        # Custom
        if color == 'custom':
            custom_color = self.choose_custom_color()
            # Cancel?
            if custom_color == None:
                return
            # Generate file color
            file_custom_color = "".join(['file://', self.generate_icon_from_template(custom_color)])
        
        for each_folder in folders:
            if each_folder.is_gone():
                continue
            
            # Get info
            folder_name = urllib.unquote(each_folder.get_uri()[7:])
            folder = Gio.File.new_for_path(folder_name)
            info = folder.query_info('metadata::custom-icon-name', 0, None)
            
            # Restore always previously, because I don't know if I set previously with custom-icon or custom-icon-name
            info.set_attribute('metadata::custom-icon', Gio.FileAttributeType.INVALID, 0)
            info.set_attribute('metadata::custom-icon-name', Gio.FileAttributeType.INVALID, 0)
            
            # Set color
            if not color == 'restore':
                if color == 'custom':
                    info.set_attribute_string('metadata::custom-icon', file_custom_color)
                else:
                    new_color = self.get_icon_name(folder_name, color)
                    info.set_attribute_string('metadata::custom-icon-name', new_color)
            
            # Write changes
            folder.set_attributes_from_info(info, 0, None)
            
            # Reload changes
            self.reload_icon(folder_name)
    
    # Nautilus invoke this function in its startup > Then, create menu entry
    def get_file_items(self, window, items_selected):
        if not len(self.get_colors()):
            return # No icons = Nothing to do
        
        # No items selected
        if not len(items_selected):
            return
        for each_item in items_selected:
            # GNOME can only handle files
            if each_item.get_uri_scheme() != 'file':
                return
            # Only folders
            if not each_item.is_directory():
                return
        
        # Main menu [1 or +1 folder(s)]
        if len(items_selected) > 1:
            top_menuitem = Nautilus.MenuItem(name='ChangeFolderColorMenu::Top', label=_("Folders Color"), icon='folder_color_picker')
        else:
            top_menuitem = Nautilus.MenuItem(name='ChangeFolderColorMenu::Top', label=_("Folder Color"),  icon='folder_color_picker')
        submenu = Nautilus.Menu()
        top_menuitem.set_submenu(submenu)
        
        # Colors submenu
        sorted_colors = sorted(self.get_colors().items(), key=lambda x:x[1])
        for color in sorted_colors:
            name = ''.join(['ChangeFolderColorMenu::"', color[0], '"'])
            label = color[1]
            item = Nautilus.MenuItem(name=name, label=label, icon=self.get_icon_name('Default', color[0].lower()))
            item.connect('activate', self.menu_activate_cb, color[0], items_selected)
            submenu.append_item(item)
        
        # Separator
        item_sep = Nautilus.MenuItem(name='ChangeFolderColorMenu::Sep1', label=self.SEPARATOR, sensitive=False)
        submenu.append_item(item_sep)
            
        # Custom color
        item_custom = Nautilus.MenuItem(name=''.join(['ChangeFolderColorMenu::"custom"']), label=_("Custom"), icon='gtk-edit')
        item_custom.connect('activate', self.menu_activate_cb, 'custom', items_selected)
        submenu.append_item(item_custom)
        
        # Restore
        item_restore = Nautilus.MenuItem(name='ChangeFolderColorMenu::Restore', label=_("Default"), icon='stock_undo')
        item_restore.connect('activate', self.menu_activate_cb, 'restore', items_selected)
        submenu.append_item(item_restore)
        
        #  & Hide Donation
        if not os.path.exists(self.HIDE_DONATION):
            # Separator
            item_sep = Nautilus.MenuItem(name='ChangeFolderColorMenu::Sep2', label=self.SEPARATOR, sensitive=False)
            submenu.append_item(item_sep)
            # Donation
            item = Nautilus.MenuItem(name=''.join(['ChangeFolderColorMenu::"donate"']), label=_("Donate?"))
            item.connect('activate', self.menu_activate_cb, 'donate', items_selected)
            submenu.append_item(item)
            # Hide donation
            item = Nautilus.MenuItem(name=''.join(['ChangeFolderColorMenu::"hideDonate"']), label=_("Hide donation"))
            item.connect('activate', self.menu_activate_cb, 'hideDonate', items_selected)
            submenu.append_item(item)
        
        return top_menuitem,
